package com.lantone.structure.facade.tran;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.structure.facade.tran.util.CommonAnalysisUtil;
import com.lantone.structure.model.doc.operation.OperationDoc;
import com.lantone.structure.model.doc.operation.OperationRecordDoc;
import com.lantone.structure.model.entity.Intervention;
import com.lantone.structure.model.entity.OperationIncision;
import com.lantone.structure.model.label.OperationRecordLabel;
import com.lantone.structure.util.MapUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Description: 一般手术记录
 * @author: wsy
 * @time: 2021/3/11 15:49
 */
@Slf4j
public class OperationRecordDocTran extends TargetTran {
    @Override
    public Map<String, String> convert(String text) {
        List<OperationDoc> operationDocs = new ArrayList<OperationDoc>();
        OperationDoc operationDoc = new OperationDoc();
        OperationRecordDoc operationRecordDoc = new OperationRecordDoc();
        operationRecordDoc.setText(text);
        Map<String, String> structureMap = cutWord(text);
        Map<String, String> retMap = new HashMap<String, String>();
        CommonAnalysisUtil.mapKeyContrastCommon(structureMap, keyContrasts, retMap);
        operationRecordDoc.setStructureMap(retMap);
        operationDoc.setOperationRecordDoc(operationRecordDoc);
        operationDocs.add(operationDoc);
        inputInfo.setOperationDocs(operationDocs);

        aiProcess();
        fetchModelData(inputInfo.getOperationDocs(), retMap);
        retMap.entrySet().removeIf(entry -> StringUtil.isBlank(entry.getValue()));
        retMap.entrySet().removeIf(entry -> !entry.getValue().matches("[\\d]+年[\\d]+月[\\d]+日[\\d]+时[\\d]+分") && "医师签名日期时间".equals(entry.getKey()));
        return retMap;
    }

    private void fetchModelData(List<OperationDoc> operationDocs, Map<String, String> retMap) {
        if (ListUtil.isNotEmpty(operationDocs)) operationDocs.forEach(operationDoc -> {
            OperationRecordDoc operationRecordDoc = operationDoc.getOperationRecordDoc();
            if (operationRecordDoc != null) {
                int interventionCount = 1;
                int operationIncisionCount = 1;
                StringBuffer interventionString = new StringBuffer();
                StringBuffer operationIncisionString = new StringBuffer();
                OperationRecordLabel operationRecordLabel = operationRecordDoc.getOperationRecordLabel();
                if (operationRecordLabel == null) {
                    return;
                }

                //介入物名称
                if (ListUtil.isNotEmpty(operationRecordLabel.getInterventions())) {
                    for (Intervention interventions : operationRecordLabel.getInterventions()) {
                        if (StringUtils.isNotEmpty(interventions.getName())) {
                            if (operationRecordLabel.getInterventions().size() > 1) {
                                interventionString.append((interventionCount++) + ".");
                            }
                            interventionString.append(interventions.getName());
                        }

                    }
                    retMap.put("介入物名称", interventionString.toString());
                }

                //皮肤消毒描述
                if (operationRecordLabel.getSkinSterilize() != null && StringUtils.isNotEmpty(operationRecordLabel.getSkinSterilize().getName())) {
                    retMap.put("皮肤消毒描述", operationRecordLabel.getSkinSterilize().getName());
                }

                //手术切口描述
                if (ListUtil.isNotEmpty(operationRecordLabel.getOperationIncision())) {
                    for (OperationIncision operationIncisions : operationRecordLabel.getOperationIncision()) {
                        if (StringUtils.isNotEmpty(operationIncisions.getName())) {
                            if (operationRecordLabel.getOperationIncision().size() > 1) {
                                operationIncisionString.append((operationIncisionCount++) + ".");
                            }
                            operationIncisionString.append(operationIncisions.getName());
                        }

                    }
                    retMap.put("手术切口描述", operationIncisionString.toString());
                }

                //输液量（mL）
                if (operationRecordLabel.getTransfuseQuantity() != null && StringUtils.isNotEmpty(operationRecordLabel.getTransfuseQuantity().getName())) {
                    retMap.put("输液量（mL）", operationRecordLabel.getTransfuseQuantity().getName());
                }

                //术前用药
                if (operationRecordLabel.getPreoperativeDrugs() != null && StringUtils.isNotEmpty(operationRecordLabel.getPreoperativeDrugs().getName())) {
                    retMap.put("术前用药", operationRecordLabel.getPreoperativeDrugs().getName());
                }

                //术中用药
                if (operationRecordLabel.getDuringDrugs() != null && StringUtils.isNotEmpty(operationRecordLabel.getDuringDrugs().getName())) {
                    retMap.put("术中用药", operationRecordLabel.getDuringDrugs().getName());
                }

                //引流材料名称
                if (operationRecordLabel.getDrainageMaterialsName() != null && StringUtils.isNotEmpty(operationRecordLabel.getDrainageMaterialsName().getName())) {
                    retMap.put("引流材料名称", operationRecordLabel.getDrainageMaterialsName().getName());
                }

                //引流材料数目
                if (operationRecordLabel.getDrainageMaterialsAmount() != null && StringUtils.isNotEmpty(operationRecordLabel.getDrainageMaterialsAmount().getName())) {
                    retMap.put("引流材料数目", operationRecordLabel.getDrainageMaterialsAmount().getName());
                }

                //放置部位
                if (operationRecordLabel.getPlacePosition() != null && StringUtils.isNotEmpty(operationRecordLabel.getPlacePosition().getName())) {
                    retMap.put("放置部位", operationRecordLabel.getPlacePosition().getName());
                }
            }
        });
    }

    private Map<String, String> cutWord(String text) {
        Map<String, String> sourceMap = Maps.newHashMap();
        text = text.replace("年  月  日  时  分", "").replace("第1次手术", "")
                .replace("（包括体位、切口处理经过，病理标本肉眼所见）", "");
        List<String> titles = CommonAnalysisUtil.sortTitles(
                Lists.newArrayList("手术开始时间", "手术结束时间", "术前诊断", "手术名称", "术中诊断", "手 术 者", "手术人员", "助手",
                        "麻醉方法", "麻醉方式", "麻 醉 者", "麻醉医师", "手术标本", "手术切除标本", "术中并发症", "术中失血量", "术中输血量", "手术简要经过（包括“术中所见”）",
                        "手术经过", "术后情况及诊疗计划", "术后注意事项", "记录者", "主刀医生", "医生签名", "患者或代理人签名", "与病人关系"),
                text
        );
        CommonAnalysisUtil.cutByTitles(text, titles, 0, sourceMap);
        if (sourceMap != null) {
            if (sourceMap.containsKey("主刀医生")) {
                String dataStr = sourceMap.get("主刀医生");
                String date = extractDate(dataStr);
                if (StringUtil.isNotBlank(date)) {
                    sourceMap.put("主刀医生", dataStr.replace(date, ""));
                    sourceMap.put("签名日期时间", date);
                }
            } else if (sourceMap.containsKey("医生签名")) {
                String dataStr = sourceMap.get("医生签名");
                String date = extractDate(dataStr);
                if (StringUtil.isNotBlank(date)) {
                    sourceMap.put("医生签名", dataStr.replace(date, ""));
                    sourceMap.put("签名日期时间", date);
                }
            }

            //其他模块使用
            if (sourceMap.containsKey("手术开始时间") && sourceMap.containsKey("手术结束时间")) {
                sourceMap.put("手术日期时间", sourceMap.get("手术开始时间") + "-" + sourceMap.get("手术结束时间"));
            }

            CommonAnalysisUtil.removeKey(sourceMap, "术前诊断", "术中诊断", "⒈患者基本情况", "⒉拟实施的输血方案", "手术标本", "手术切除标本", "术中并发症",
                    "与病人关系", "术后情况及诊疗计划", "术后注意事项", "手术简要经过（包括“术中所见”）");
        }
        return sourceMap;
    }

    /**
     * 抽取文本中的第一个时间
     *
     * @param top
     * @return
     */
    private String extractDate(String top) {
        Pattern pattern = Pattern.compile("[0-9]{4}[-][0-9]{1,2}[-][0-9]{1,2}");
        Matcher matcher = pattern.matcher(top);
        if (matcher.find()) {
            return matcher.group(0);
        } else {
            Pattern p1 = Pattern.compile("[0-9]{4}年[0-9]+月[0-9]+日 [0-9]+时[0-9]+分");
            Matcher m1 = p1.matcher(top);
            if (m1.find()) {
                return m1.group(0);
            }
        }
        return null;
    }

    private void disDate(Map sourceMap, String... args) {
        Map<String, String> sourceMap_ = MapUtil.copyMap(sourceMap);
        if (sourceMap.containsKey(args[0]) && sourceMap_.get(args[0]).contains(args[1])) {
            int index = sourceMap_.get(args[0]).indexOf(args[1]);
            sourceMap.put(args[0], sourceMap_.get(args[0]).substring(0, index));
            sourceMap.put(args[0] + args[1], sourceMap_.get(args[0]).substring(index).replace(args[1], "").replaceAll("[:：]", ""));
        }
    }


    private List<String> keyContrasts = Lists.newArrayList(
            "手术开始时间=手术开始日期时间",
            "手术结束时间=手术结束日期时间",
            "手术经过=手术过程描述",
            "手术者=手术者姓名",
            "手术人员=手术者姓名",
            "麻醉者=麻醉医师姓名",
            "麻醉医师=麻醉医师姓名",
            "医护人员陈述=手术者签名",
            "主刀医生=手术者签名",
            "术中失血量=出血量（mL）",
            "术中输血量=输血量（mL）"
    );
}